/*
 * AF tester
 * Copyright 2004, Clemens Fruhwirth <clemens@endorphin.org>
 * Copyright 2015, Sami Farin <sami.farin@gmail.com>
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "af.h"
#include "charcrandom.h"

int main(int argc, char **argv)
{
	#define SIZE 64
	//#define TIMES ((8<<20)/SIZE)
	#define TIMES 3
	char data[SIZE];
	char recover[SIZE];
	char *buf;
	size_t bpos;
	int ret;
	uint8_t corrupt;
	int testfail = 0;

	buf = calloc(SIZE, TIMES);
	if (buf == NULL) {
		printf("out of memory\n");
		return 1;
	}
	charcrandom_buf(data, SIZE);
	ret = AF_split(data, buf, SIZE, TIMES);
	if (ret < 0) {
		printf("AF_split failed: %s\n", strerror(-ret));
		return 1;
	}
#if 1
	write(666, data, SIZE);
	write(667, buf, SIZE*TIMES);
#endif
	ret = AF_merge(buf, recover, SIZE, TIMES);
	if (ret < 0) {
		printf("AF_merge 1 failed: %s\n", strerror(-ret));
		return 1;
	}
	if (memcmp(data, recover, SIZE))
		testfail = 1;
	printf("Data %sequal, should be equal\n\n",
		testfail ? "not " : "");

	AF_split(data, buf, SIZE, TIMES);
	/* 1-8 bit error */
	bpos = charcrandom_uniform64(SIZE*TIMES);
	do {
		charcrandom_buf(&corrupt, sizeof(corrupt));
	} while(corrupt == 0);
	buf[bpos] ^= corrupt;
	ret = AF_merge(buf, recover, SIZE, TIMES);
	if (ret < 0) {
		printf("AF_merge 2 failed: %s\n", strerror(-ret));
		return 1;
	}
	if (!memcmp(data, recover, SIZE))
		testfail = 1;
	printf("Data %sequal, should be not equal (bytepos=%zu xor=%u)\n",
		testfail ? "" : "not ", bpos, corrupt);

	return testfail;
}

